@@ -23,7 +23,7 @@ import re, os
 
 
 # The current version of rdiff-backup
-version = "0.13.1"
+version = "0.13.2"
 
 # If this is set, use this value in seconds as the current time
 # instead of reading it from the clock.
@@ -53,9 +53,6 @@ process_gid = os.getgid()
 # If true, when copying attributes, also change target's uid/gid
 change_ownership = None
 
-# If true, when copying attributes, also change target's permission.
-change_permissions = 1
-
 # If true, change the permissions of unwriteable mirror files
 # (such as directories) so that they can be written, and then
 # change them back.  This defaults to 1 just in case the process
@@ -124,6 +121,12 @@ backup_writer = None
 # Connection of the client
 client_conn = None
 
+# When backing up, issource should be true on the reader and isdest on
+# the writer.  When restoring, issource should be true on the mirror
+# and isdest should be true on the target.
+issource = None
+isdest = None
+
 # This list is used by the set function below.  When a new
 # connection is created with init_connection, its Globals class
 # will match this one for all the variables mentioned in this
@@ -204,10 +207,8 @@ compare_inode = 1
 # guarantee that any changes have been committed to disk.
 fsync_directories = 1
 
-# If set, directory increments are given the same permissions as the
-# directories they represent.  Otherwise they have the default
-# permissions.
-change_dir_inc_perms = 1
+# If set, exit with error instead of dropping ACLs or ACL entries.
+never_drop_acls = None
 
 
 def get(name):
