@@ -32,6 +32,7 @@ remote_cmd, remote_schema = None, None
 force = None
 select_opts = []
 select_files = []
+user_mapping_filename, group_mapping_filename = None, None
 # These are global because they are set while we are trying to figure
 # whether to restore or to backup
 restore_root, restore_index, restore_root_set = None, None, 0
@@ -40,6 +41,7 @@ def parse_cmdlineoptions(arglist):
 	"""Parse argument list and set global preferences"""
 	global args, action, force, restore_timestr, remote_cmd, remote_schema
 	global remove_older_than_string
+	global user_mapping_filename, group_mapping_filename
 	def sel_fl(filename):
 		"""Helper function for including/excluding filelists below"""
 		try: return open(filename, "r")
@@ -51,12 +53,12 @@ def parse_cmdlineoptions(arglist):
 		  "exclude-filelist=", "exclude-filelist-stdin",
 		  "exclude-globbing-filelist=", "exclude-mirror=",
 		  "exclude-other-filesystems", "exclude-regexp=",
-		  "exclude-special-files", "force", "include=",
-		  "include-filelist=", "include-filelist-stdin",
+		  "exclude-special-files", "force", "group-mapping-file=",
+		  "include=", "include-filelist=", "include-filelist-stdin",
 		  "include-globbing-filelist=", "include-regexp=",
 		  "list-at-time=", "list-changed-since=", "list-increments",
-		  "list-increment-sizes", "no-compare-inode",
-		  "no-change-dir-inc-perms", "no-compression",
+		  "list-increment-sizes", "never-drop-acls",
+		  "no-compare-inode", "no-compression",
 		  "no-compression-regexp=", "no-file-statistics",
 		  "no-hard-links", "null-separator",
 		  "override-chars-to-quote=", "parsable-output",
@@ -64,7 +66,7 @@ def parse_cmdlineoptions(arglist):
 		  "remove-older-than=", "restore-as-of=", "restrict=",
 		  "restrict-read-only=", "restrict-update-only=", "server",
 		  "ssh-no-compression", "terminal-verbosity=", "test-server",
-		  "verbosity=", "version"])
+		  "user-mapping-file=", "verbosity=", "version"])
 	except getopt.error, e:
 		commandline_error("Bad commandline options: %s" % str(e))
 
@@ -89,6 +91,7 @@ def parse_cmdlineoptions(arglist):
 			  opt == "--exclude-regexp" or
 			  opt == "--exclude-special-files"): select_opts.append((opt, arg))
 		elif opt == "--force": force = 1
+		elif opt == "--group-mapping-file": group_mapping_filename = arg
 		elif opt == "--include": select_opts.append((opt, arg))
 		elif opt == "--include-filelist":
 			select_opts.append((opt, arg))
@@ -107,8 +110,7 @@ def parse_cmdlineoptions(arglist):
 		elif opt == "-l" or opt == "--list-increments":
 			action = "list-increments"
 		elif opt == '--list-increment-sizes': action = 'list-increment-sizes'
-		elif opt == '--no-change-dir-inc-perms':
-			Globals.set('change_dir_inc_perms', 0)
+		elif opt == "--never-drop-acls": Globals.set("never_drop_acls", 1)
 		elif opt == "--no-compare-inode": Globals.set("compare_inode", 0)
 		elif opt == "--no-compression": Globals.set("compression", None)
 		elif opt == "--no-compression-regexp":
@@ -141,6 +143,7 @@ def parse_cmdlineoptions(arglist):
 			Globals.set('ssh_compression', None)
 		elif opt == "--terminal-verbosity": Log.setterm_verbosity(arg)
 		elif opt == "--test-server": action = "test-server"
+		elif opt == "--user-mapping-file": user_mapping_filename = arg
 		elif opt == "-V" or opt == "--version":
 			print "rdiff-backup " + Globals.version
 			sys.exit(0)
@@ -156,11 +159,15 @@ def check_action():
 						   'list-changed-since', 'check-destination-dir'],
 					   2: ['backup', 'restore', 'restore-as-of']}
 	l = len(args)
-	if not action: assert l == 2, args # cannot tell backup or restore yet
-	elif action == 'calculate-average':
-		if l == 0: commandline_error("No arguments given")
+	if l == 0 and action not in arg_action_dict[l]:
+		commandline_error("No arguments given")
+	elif not action:
+		if l == 2: pass # Will determine restore or backup later
+		else:
+			commandline_error("Switches missing or wrong number of arguments")
+	elif action == 'test-server': pass # test-server takes any number of args
 	elif l > 2 or action not in arg_action_dict[l]:
-		commandline_error("Wrong number of arguments given.  See man page.")
+		commandline_error("Wrong number of arguments given.")
 
 def final_set_action(rps):
 	"""If no action set, decide between backup and restore at this point"""
@@ -186,6 +193,21 @@ def misc_setup(rps):
 		conn.robust.install_signal_handlers()
 		conn.Hardlink.initialize_dictionaries()
 
+def init_user_group_mapping(destination_conn):
+	"""Initialize user and group mapping on destination connection"""
+	global user_mapping_filename, group_mapping_filename
+	def get_string_from_file(filename):
+		if not filename: return None
+		rp = rpath.RPath(Globals.local_connection, filename)
+		try: return rp.get_data()
+		except OSError, e:
+			log.FatalError("Error '%s' reading mapping file '%s'" %
+						   (str(e), filename))
+	user_mapping_string = get_string_from_file(user_mapping_filename)
+	destination_conn.user_group.init_user_mapping(user_mapping_string)
+	group_mapping_string = get_string_from_file(group_mapping_filename)
+	destination_conn.user_group.init_group_mapping(group_mapping_string)
+
 def take_action(rps):
 	"""Do whatever action says"""
 	if action == "server":
@@ -226,14 +248,14 @@ def Main(arglist):
 
 def Backup(rpin, rpout):
 	"""Backup, possibly incrementally, src_path to dest_path."""
+	global incdir
 	SetConnections.BackupInitConnections(rpin.conn, rpout.conn)
 	backup_check_dirs(rpin, rpout)
-	backup_set_fs_globals(rpin, rpout)
-	if Globals.chars_to_quote:
-		rpout = FilenameMapping.get_quotedrpath(rpout)
-		SetConnections.UpdateGlobal(
-			'rbdir', FilenameMapping.get_quotedrpath(Globals.rbdir))
 	backup_set_rbdir(rpin, rpout)
+	backup_set_fs_globals(rpin, rpout)
+	if Globals.chars_to_quote: rpout = backup_quoted_rpaths(rpout)
+	backup_final_init(rpout)
+	init_user_group_mapping(rpout.conn)
 	backup_set_select(rpin)
 	if prevtime:
 		rpout.conn.Main.backup_touch_curmirror_local(rpin, rpout)
@@ -244,6 +266,14 @@ def Backup(rpin, rpout):
 		backup.Mirror(rpin, rpout)
 		rpout.conn.Main.backup_touch_curmirror_local(rpin, rpout)
 
+def backup_quoted_rpaths(rpout):
+	"""Get QuotedRPath versions of important RPaths.  Return rpout"""
+	global incdir
+	SetConnections.UpdateGlobal(
+		'rbdir', FilenameMapping.get_quotedrpath(Globals.rbdir))
+	incdir = FilenameMapping.get_quotedrpath(incdir)
+	return FilenameMapping.get_quotedrpath(rpout)
+
 def backup_set_select(rpin):
 	"""Create Select objects on source connection"""
 	rpin.conn.backup.SourceStruct.set_source_select(rpin, select_opts,
@@ -271,16 +301,13 @@ def backup_check_dirs(rpin, rpout):
 
 def backup_set_rbdir(rpin, rpout):
 	"""Initialize data dir and logging"""
-	global incdir, prevtime
+	global incdir
 	SetConnections.UpdateGlobal('rbdir', Globals.rbdir)
-	checkdest_if_necessary(rpout)
 	incdir = Globals.rbdir.append_path("increments")
-	prevtime = backup_get_mirrortime()
 
 	assert rpout.lstat(), (rpout.path, rpout.lstat())
 	if rpout.isdir() and not rpout.listdir(): # rpout is empty dir
-		if Globals.change_permissions:
-			rpout.chmod(0700) # just make sure permissions aren't too lax
+		rpout.chmod(0700) # just make sure permissions aren't too lax
 	elif not Globals.rbdir.lstat() and not force: Log.FatalError(
 """Destination directory
 
@@ -292,11 +319,6 @@ want to update or overwrite it, run rdiff-backup with the --force
 option.""" % rpout.path)
 
 	if not Globals.rbdir.lstat(): Globals.rbdir.mkdir()
-	inc_base = Globals.rbdir.append_path("increments")
-	if not inc_base.lstat(): inc_base.mkdir()
-	if Log.verbosity > 0:
-		Log.open_logfile(Globals.rbdir.append("backup.log"))
-	ErrorLog.open(Time.curtimestr, compress = Globals.compression)
 
 def backup_warn_if_infinite_regress(rpin, rpout):
 	"""Warn user if destination area contained in source area"""
@@ -321,6 +343,21 @@ def backup_get_mirrortime():
 	if mirror_rps: return mirror_rps[0].getinctime()
 	else: return None
 
+def backup_final_init(rpout):
+	"""Open the backup log and the error log, create increments dir"""
+	global prevtime
+	prevtime = backup_get_mirrortime()
+	need_check = checkdest_need_check(rpout)
+	if Log.verbosity > 0:
+		Log.open_logfile(Globals.rbdir.append("backup.log"))
+	ErrorLog.open(Time.curtimestr, compress = Globals.compression)
+	if need_check:
+		Log("Previous backup seems to have failed, regressing "
+			"destination now.", 2)
+		rpout.conn.regress.Regress(rpout)
+	inc_base = Globals.rbdir.append_path("increments")
+	if not inc_base.lstat(): inc_base.mkdir()
+
 def backup_set_fs_globals(rpin, rpout):
 	"""Use fs_abilities to set the globals that depend on filesystem"""
 	def update_bool_global(attr, bool):
@@ -328,11 +365,14 @@ def backup_set_fs_globals(rpin, rpout):
 		if Globals.get(attr) is None:
 			SetConnections.UpdateGlobal(attr, bool)
 
-	src_fsa = fs_abilities.FSAbilities('source').init_readonly(rpin)
+	src_fsa = rpin.conn.fs_abilities.get_fsabilities_readonly('source', rpin)
 	Log(str(src_fsa), 3)
-	dest_fsa = fs_abilities.FSAbilities('destination').init_readwrite(
-		Globals.rbdir, override_chars_to_quote = Globals.chars_to_quote)
+	dest_fsa = rpout.conn.fs_abilities.get_fsabilities_readwrite(
+		'destination', Globals.rbdir, 1, Globals.chars_to_quote)
 	Log(str(dest_fsa), 3)
+	if Globals.never_drop_acls and not dest_fsa.acls:
+		Log.FatalError("--never-drop-acls specified, but ACL support\n"
+					   "disabled on destination filesystem")
 
 	update_bool_global('read_acls', src_fsa.acls)
 	update_bool_global('read_eas', src_fsa.eas)
@@ -345,7 +385,6 @@ def backup_set_fs_globals(rpin, rpout):
 	update_bool_global('write_eas', Globals.read_eas and dest_fsa.eas)
 	update_bool_global('write_resource_forks',
 					   Globals.read_resource_forks and dest_fsa.resource_forks)
-	update_bool_global('change_dir_inc_perms', dest_fsa.dir_inc_perms)
 	SetConnections.UpdateGlobal('chars_to_quote', dest_fsa.chars_to_quote)
 	if Globals.chars_to_quote:
 		for conn in Globals.connections:
@@ -394,11 +433,12 @@ def Restore(src_rp, dest_rp, restore_as_of = None):
 	restore_set_fs_globals(dest_rp)
 	src_rp = restore_init_quoting(src_rp)
 	restore_check_backup_dir(restore_root, src_rp, restore_as_of)
+	inc_rpath = Globals.rbdir.append_path('increments', restore_index)
 	if restore_as_of:
-		try: time = Time.genstrtotime(restore_timestr)
+		try: time = Time.genstrtotime(restore_timestr, rp = inc_rpath)
 		except Time.TimeException, exc: Log.FatalError(str(exc))
 	else: time = src_rp.getinctime()
-	inc_rpath = Globals.rbdir.append_path('increments', restore_index)
+	init_user_group_mapping(dest_rp.conn)
 	restore_set_select(restore_root, dest_rp)
 	restore_start_log(src_rp, dest_rp, time)
 	restore.Restore(restore_root.new_index(restore_index),
@@ -421,10 +461,15 @@ def restore_set_fs_globals(target):
 		"""If bool is not None, update Globals.attr accordingly"""
 		if Globals.get(attr) is None: SetConnections.UpdateGlobal(attr, bool)
 
-	target_fsa = fs_abilities.FSAbilities('destination').init_readwrite(
-		target, 0)
-	mirror_fsa = fs_abilities.FSAbilities('source').init_readwrite(
+	target_fsa = target.conn.fs_abilities.get_fsabilities_readwrite(
+		'destination', target, 0)
+	Log(str(target_fsa), 3)
+	mirror_fsa = Globals.rbdir.conn.fs_abilities.get_fsabilities_restoresource(
 		Globals.rbdir)
+	Log(str(mirror_fsa), 3)
+	if Globals.never_drop_acls and not target_fsa.acls:
+		Log.FatalError("--never-drop-acls specified, but ACL support\n"
+					   "disabled on destination filesystem")
 
 	update_bool_global('read_acls', target_fsa.acls)
 	update_bool_global('write_acls', target_fsa.acls)
@@ -456,7 +501,8 @@ def restore_set_select(mirror_rp, target):
 def restore_start_log(rpin, target, time):
 	"""Open restore log file, log initial message"""
 	try: Log.open_logfile(Globals.rbdir.append("restore.log"))
-	except LoggerError, e: Log("Warning, " + str(e), 2)
+	except (LoggerError, Security.Violation), e:
+		Log("Warning - Unable to open logfile: " + str(e), 2)
 
 	# Log following message at file verbosity 3, but term verbosity 4
 	log_message = ("Starting restore of %s to %s as it was as of %s." %
@@ -507,11 +553,15 @@ def restore_set_root(rpin):
 	global restore_root, restore_index, restore_root_set
 	if rpin.isincfile(): relpath = rpin.getincbase().path
 	else: relpath = rpin.path
-	pathcomps = os.path.join(rpin.conn.os.getcwd(), relpath).split("/")
-	assert len(pathcomps) >= 2 # path should be relative to /
+	if rpin.conn is not Globals.local_connection:
+		# For security checking consistency, don't get absolute path
+		pathcomps = relpath.split('/')
+	else: pathcomps = os.path.join(os.getcwd(), relpath).split("/")
+	if not pathcomps[0]: min_len_pathcomps = 2 # treat abs paths differently
+	else: min_len_pathcomps = 1
 
 	i = len(pathcomps)
-	while i >= 2:
+	while i >= min_len_pathcomps:
 		parent_dir = rpath.RPath(rpin.conn, "/".join(pathcomps[:i]))
 		if (parent_dir.isdir() and
 			"rdiff-backup-data" in parent_dir.listdir()): break
@@ -569,7 +619,7 @@ def CalculateAverage(rps):
 
 def RemoveOlderThan(rootrp):
 	"""Remove all increment files older than a certain time"""
-	rom_check_dir(rootrp)
+	rot_check_dir(rootrp)
 	try: time = Time.genstrtotime(remove_older_than_string)
 	except Time.TimeException, exc: Log.FatalError(str(exc))
 	timep = Time.timetopretty(time)
@@ -594,7 +644,7 @@ def RemoveOlderThan(rootrp):
 	else: Log("Deleting increments at times:\n" + inc_pretty_time, 3)
 	manage.delete_earlier_than(Globals.rbdir, time)
 
-def rom_check_dir(rootrp):
+def rot_check_dir(rootrp):
 	"""Check destination dir before RemoveOlderThan"""
 	SetConnections.UpdateGlobal('rbdir',
 								rootrp.append_path("rdiff-backup-data"))
@@ -606,9 +656,9 @@ def rom_check_dir(rootrp):
 
 def ListChangedSince(rp):
 	"""List all the files under rp that have changed since restoretime"""
+	assert restore_set_root(rp)
 	try: rest_time = Time.genstrtotime(restore_timestr)
 	except Time.TimeException, exc: Log.FatalError(str(exc))
-	assert restore_set_root(rp)
 	restore_check_backup_dir(restore_root)
 	mirror_rp = restore_root.new_index(restore_index)
 	inc_rp = mirror_rp.append_path("increments", restore_index)
@@ -617,9 +667,9 @@ def ListChangedSince(rp):
 
 def ListAtTime(rp):
 	"""List files in archive under rp that are present at restoretime"""
+	assert restore_set_root(rp)
 	try: rest_time = Time.genstrtotime(restore_timestr)
 	except Time.TimeException, exc: Log.FatalError(str(exc))
-	assert restore_set_root(rp)
 	restore_check_backup_dir(restore_root)
 	mirror_rp = restore_root.new_index(restore_index)
 	inc_rp = mirror_rp.append_path("increments", restore_index)
@@ -639,6 +689,7 @@ def CheckDest(dest_rp):
 	elif need_check == 0:
 		Log.FatalError("Destination dir %s does not need checking" %
 					   (dest_rp.path,))
+	init_user_group_mapping(dest_rp.conn)
 	dest_rp.conn.regress.Regress(dest_rp)
 
 def checkdest_need_check(dest_rp):
@@ -656,8 +707,14 @@ def checkdest_need_check(dest_rp):
 The rdiff-backup data directory
 %s
 exists, but we cannot find a valid current_mirror marker.  You can
-avoid this message by removing this directory; however any data in it
-will be lost.
+avoid this message by removing the rdiff_backup_data directory;
+however any data in it will be lost.
+
+Probably this error was caused because the first rdiff-backup session
+into a new directory failed.  If this is the case it is safe to delete
+the rdiff_backup_data directory because there is no important
+information in it.
+
 """ % (Globals.rbdir.path,))
 	elif len(curmir_incs) == 1: return 0
 	else:
