@@ -1,4 +1,4 @@
-# Copyright 2002 Ben Escoto
+# Copyright 2002, 2003 Ben Escoto
 #
 # This file is part of rdiff-backup.
 #
@@ -25,9 +25,9 @@ the related connections.
 
 """
 
-import os, sys
+import os
 from log import Log
-import Globals, FilenameMapping, connection, rpath
+import Globals, connection, rpath
 
 # This is the schema that determines how rdiff-backup will open a
 # pipe to the remote system.  If the file is given as A::B, %s will
@@ -178,7 +178,6 @@ def init_connection_settings(conn):
 	conn.log.Log.setterm_verbosity(Log.term_verbosity)
 	for setting_name in Globals.changed_settings:
 		conn.Globals.set(setting_name, Globals.get(setting_name))
-	FilenameMapping.set_init_quote_vals()
 
 def init_connection_remote(conn_number):
 	"""Run on server side to tell self that have given conn_number"""
@@ -203,8 +202,6 @@ def BackupInitConnections(reading_conn, writing_conn):
 	writing_conn.Globals.set("isbackup_writer", 1)
 	UpdateGlobal("backup_reader", reading_conn)
 	UpdateGlobal("backup_writer", writing_conn)
-	if writing_conn.os.getuid() == 0 and Globals.change_ownership != 0:
-		UpdateGlobal('change_ownership', 1)
 
 def CloseConnections():
 	"""Close all connections.  Run by client"""
@@ -226,20 +223,15 @@ def test_connection(conn_number):
 	print "Testing server started by: ", __conn_remote_cmds[conn_number]
 	conn = Globals.connections[conn_number]
 	try:
-		assert conn.Globals.get('current_time') is None
-		assert type(conn.os.getuid()) is int
-		version = conn.Globals.get('version')
+		assert conn.pow(2,3) == 8
+		assert conn.os.path.join("a", "b") == "a/b"
+		version = conn.reval("lambda: Globals.version")
 	except:
 		sys.stderr.write("Server tests failed\n")
 		raise
 	if not version == Globals.version:
 		print """Server may work, but there is a version mismatch:
 Local version: %s
-Remote version: %s
-
-In general, an attempt is made to guarantee compatibility only between
-different minor versions of the same stable series.  For instance, you
-should expect 0.12.4 and 0.12.7 to be compatible, but not 0.12.7
-and 0.13.3, nor 0.13.2 and 0.13.4.
-""" % (Globals.version, version)
+Remote version: %s""" % (Globals.version, version)
 	else: print "Server OK"
+
