@@ -27,6 +27,7 @@ import Globals, metadata, rorpiter, TempFile, Hardlink, robust, increment, \
 
 def Mirror(src_rpath, dest_rpath):
 	"""Turn dest_rpath into a copy of src_rpath"""
+	log.Log("Starting mirror %s to %s" % (src_rpath.path, dest_rpath.path), 4)
 	SourceS = src_rpath.conn.backup.SourceStruct
 	DestS = dest_rpath.conn.backup.DestinationStruct
 
@@ -38,6 +39,8 @@ def Mirror(src_rpath, dest_rpath):
 
 def Mirror_and_increment(src_rpath, dest_rpath, inc_rpath):
 	"""Mirror + put increments in tree based at inc_rpath"""
+	log.Log("Starting increment operation %s to %s" %
+			(src_rpath.path, dest_rpath.path), 4)
 	SourceS = src_rpath.conn.backup.SourceStruct
 	DestS = dest_rpath.conn.backup.DestinationStruct
 
@@ -146,7 +149,7 @@ class DestinationStruct:
 		dest_iter = cls.get_dest_select(baserp, for_increment)
 		collated = rorpiter.Collate2Iters(source_iter, dest_iter)
 		cls.CCPP = CacheCollatedPostProcess(
-			collated, Globals.pipeline_max_length*4)
+			collated, Globals.pipeline_max_length*4, baserp)
 		# pipeline len adds some leeway over just*3 (to and from and back)
 		
 	def get_sigs(cls, dest_base_rpath):
@@ -184,15 +187,26 @@ class DestinationStruct:
 		elif dest_rorp:
 			dest_sig = dest_rorp.getRORPath()
 			if dest_rorp.isreg():
-				dest_rp = dest_base_rpath.new_index(index)
-				if not dest_rp.isreg():
-					log.ErrorLog.write_if_open("UpdateError", dest_rp,
-						"File changed from regular file before signature")
-					return None
-				dest_sig.setfile(Rdiff.get_signature(dest_rp))
+				sig_fp = cls.get_one_sig_fp(dest_base_rpath.new_index(index))
+				if sig_fp is None: return None
+				dest_sig.setfile(sig_fp)
 		else: dest_sig = rpath.RORPath(index)
 		return dest_sig			
 
+	def get_one_sig_fp(cls, dest_rp):
+		"""Return a signature fp of given index, corresponding to reg file"""
+		if not dest_rp.isreg():
+			log.ErrorLog.write_if_open("UpdateError", dest_rp,
+				"File changed from regular file before signature")
+			return None
+		if Globals.process_uid != 0 and not dest_rp.readable():
+			# This branch can happen with root source and non-root
+			# destination.  Permissions are changed permanently, which
+			# should propogate to the diffs
+			assert dest_rp.isowner(), 'no ownership of %s' % (dest_rp.path,)
+			dest_rp.chmod(0400 | dest_rp.getperms())
+		return Rdiff.get_signature(dest_rp)
+				
 	def patch(cls, dest_rpath, source_diffiter, start_index = ()):
 		"""Patch dest_rpath with an rorpiter of diffs"""
 		ITR = rorpiter.IterTreeReducer(PatchITRB, [dest_rpath, cls.CCPP])
@@ -218,9 +232,9 @@ static.MakeClass(DestinationStruct)
 
 
 class CacheCollatedPostProcess:
-	"""Cache a collated iter of (source_rorp, dest_rp) pairs
+	"""Cache a collated iter of (source_rorp, dest_rorp) pairs
 
-	This is necessary for two reasons:
+	This is necessary for three reasons:
 
 	1.  The patch function may need the original source_rorp or
 	    dest_rp information, which is not present in the diff it
@@ -233,6 +247,11 @@ class CacheCollatedPostProcess:
 	    any metadata until we know the file has been procesed
 	    correctly.
 
+	3.  We may lack permissions on certain destination directories.
+	    The permissions of these directories need to be relaxed before
+	    we enter them to computer signatures, and then reset after we
+	    are done patching everything inside them.
+
 	The class caches older source_rorps and dest_rps so the patch
 	function can retrieve them if necessary.  The patch function can
 	also update the processed correctly flag.  When an item falls out
@@ -240,10 +259,12 @@ class CacheCollatedPostProcess:
 	metadata for it.
 
 	"""
-	def __init__(self, collated_iter, cache_size):
+	def __init__(self, collated_iter, cache_size, dest_root_rp):
 		"""Initialize new CCWP."""
 		self.iter = collated_iter # generates (source_rorp, dest_rorp) pairs
 		self.cache_size = cache_size
+		self.dest_root_rp = dest_root_rp
+
 		self.statfileobj = statistics.init_statfileobj()
 		if Globals.file_statistics: statistics.FileStats.init()
 		metadata.MetadataFile.open_file()
@@ -265,6 +286,11 @@ class CacheCollatedPostProcess:
 		self.cache_dict = {}
 		self.cache_indicies = []
 
+		# Contains a list of pairs (destination_rps, permissions) to
+		# be used to reset the permissions of certain directories
+		# after we're finished with them
+		self.dir_perms_list = []
+
 	def __iter__(self): return self
 
 	def next(self):
@@ -288,6 +314,21 @@ class CacheCollatedPostProcess:
 		"""
 		if source_rorp: Hardlink.add_rorp(source_rorp, source = 1)
 		if dest_rorp: Hardlink.add_rorp(dest_rorp, source = 0)
+		if (dest_rorp and dest_rorp.isdir() and Globals.process_uid != 0
+			and dest_rorp.getperms() % 01000 < 0700):
+			self.unreadable_dir_init(source_rorp, dest_rorp)
+
+	def unreadable_dir_init(self, source_rorp, dest_rorp):
+		"""Initialize an unreadable dir.
+
+		Make it readable, and if necessary, store the old permissions
+		in self.dir_perms_list so the old perms can be restored.
+
+		"""
+		dest_rp = self.dest_root_rp.new_index(dest_rorp.index)
+		dest_rp.chmod(0700 | dest_rorp.getperms())
+		if source_rorp and source_rorp.isdir():
+			self.dir_perms_list.append((dest_rp, source_rorp.getperms()))
 
 	def shorten_cache(self):
 		"""Remove one element from cache, possibly adding it to metadata"""
@@ -302,6 +343,7 @@ class CacheCollatedPostProcess:
 		del self.cache_dict[first_index]
 		self.post_process(old_source_rorp, old_dest_rorp,
 						  changed_flag, success_flag, inc)
+		if self.dir_perms_list: self.reset_dir_perms(first_index)
 
 	def post_process(self, source_rorp, dest_rorp, changed, success, inc):
 		"""Post process source_rorp and dest_rorp.
@@ -332,6 +374,14 @@ class CacheCollatedPostProcess:
 		if Globals.file_statistics:
 			statistics.FileStats.update(source_rorp, dest_rorp, changed, inc)
 
+	def reset_dir_perms(self, current_index):
+		"""Reset the permissions of directories when we have left them"""
+		dir_rp, perms = self.dir_perms_list[-1]
+		dir_index = dir_rp.index
+		if (current_index > dir_index and
+			current_index[:len(dir_index)] != dir_index):
+			dir_rp.chmod(perms) # out of directory, reset perms now
+
 	def in_cache(self, index):
 		"""Return true if given index is cached"""
 		return self.cache_dict.has_key(index)
@@ -370,6 +420,9 @@ class CacheCollatedPostProcess:
 	def close(self):
 		"""Process the remaining elements in the cache"""
 		while self.cache_indicies: self.shorten_cache()
+		while self.dir_perms_list:
+			dir_rp, perms = self.dir_perms_list.pop()
+			dir_rp.chmod(perms)
 		metadata.MetadataFile.close_file()
 		if Globals.read_eas: eas_acls.ExtendedAttributesFile.close_file()
 		if Globals.read_acls: eas_acls.AccessControlListFile.close_file()
@@ -479,6 +532,7 @@ class PatchITRB(rorpiter.ITRBranch):
 		"""Set self.dir_replacement, which holds data until done with dir
 
 		This is used when base_rp is a dir, and diff_rorp is not.
+		Returns 1 for success or 0 for failure
 
 		"""
 		assert diff_rorp.get_attached_filetype() == 'snapshot'
@@ -488,10 +542,8 @@ class PatchITRB(rorpiter.ITRBranch):
 			# Was an error, so now restore original directory
 			rpath.copy_with_attribs(self.CCPP.get_mirror_rorp(diff_rorp.index),
 									self.dir_replacement)
-			success = 0
-		else: success = 1
-		if base_rp.isdir() and Globals.change_permissions: base_rp.chmod(0700)
-		return success
+			return 0
+		else: return 1
 
 	def prepare_dir(self, diff_rorp, base_rp):
 		"""Prepare base_rp to turn into a directory"""
@@ -503,7 +555,6 @@ class PatchITRB(rorpiter.ITRBranch):
 		else: # maybe no change, so query CCPP before tagging success
 			if self.CCPP.in_cache(diff_rorp.index):
 				self.CCPP.flag_success(diff_rorp.index)
-		if Globals.change_permissions: base_rp.chmod(0700)
 
 	def end_process(self):
 		"""Finish processing directory"""
@@ -582,3 +633,4 @@ class IncrementITRB(PatchITRB):
 				self.CCPP.set_inc(index, inc)
 				self.CCPP.flag_success(index)
 
+
