@@ -27,9 +27,9 @@ metadata file (which we assume is intact) and delete the extra
 increments.  For regular files we examine the mirror file and use the
 increment file to get the old data if the mirror file is out of date.
 
-Currently this does recover hard links.  This make make the regressed
-directory take up more disk space, but hard links can still be
-recovered.
+Currently this does not recover hard links.  This may make the
+regressed directory take up more disk space, but hard links can still
+be recovered.
 
 """
 
@@ -249,8 +249,7 @@ class RegressITRB(rorpiter.ITRBranch):
 			if not rf.mirror_rp.isdir():
 				if rf.mirror_rp.lstat(): rf.mirror_rp.delete()
 				rf.mirror_rp.mkdir()
-			if Globals.change_permissions and not rf.mirror_rp.hasfullperms():
-				rf.mirror_rp.chmod(0700)
+			if not rf.mirror_rp.hasfullperms(): rf.mirror_rp.chmod(0700)
 		self.rf = rf
 
 	def end_process(self):
