@@ -36,7 +36,7 @@ are dealing with are local or remote.
 """
 
 import os, stat, re, sys, shutil, gzip, socket, time
-import Globals, Time, static, log
+import Globals, Time, static, log, user_group
 
 
 class SkipFileException(Exception):
@@ -143,7 +143,7 @@ def cmp(rpin, rpout):
 	elif rpin.issock(): return rpout.issock()
 	else: raise RPathException("File %s has unknown type" % rpin.path)
 
-def copy_attribs(rpin, rpout, acls = 1):
+def copy_attribs(rpin, rpout):
 	"""Change file attributes of rpout to match rpin
 
 	Only changes the chmoddable bits, uid/gid ownership, and
@@ -151,14 +151,35 @@ def copy_attribs(rpin, rpout, acls = 1):
 
 	"""
 	log.Log("Copying attributes from %s to %s" % (rpin.index, rpout.path), 7)
+	assert rpin.lstat() == rpout.lstat() is not None, "different file types"
+	if rpin.issym(): return # symlinks have no valid attributes
+	if Globals.write_resource_forks and rpin.isreg():
+		rpout.write_resource_fork(rpin.get_resource_fork())
+	if Globals.write_eas: rpout.write_ea(rpin.get_ea())
+	if Globals.change_ownership: rpout.chown(*user_group.map_rpath(rpin))
+	rpout.chmod(rpin.getperms())
+	if Globals.write_acls: rpout.write_acl(rpin.get_acl())
+	if not rpin.isdev(): rpout.setmtime(rpin.getmtime())
+
+def copy_attribs_inc(rpin, rpout):
+	"""Change file attributes of rpout to match rpin
+
+	Like above, but used to give increments the same attributes as the
+	originals.  Therefore, don't copy all directory acl and
+	permissions.
+
+	"""
+	log.Log("Copying inc attrs from %s to %s" % (rpin.index, rpout.path), 7)
 	check_for_files(rpin, rpout)
 	if rpin.issym(): return # symlinks have no valid attributes
 	if Globals.write_resource_forks and rpin.isreg() and rpout.isreg():
 		rpout.write_resource_fork(rpin.get_resource_fork())
 	if Globals.write_eas: rpout.write_ea(rpin.get_ea())
 	if Globals.change_ownership: apply(rpout.chown, rpin.getuidgid())
-	if Globals.change_permissions: rpout.chmod(rpin.getperms())
-	if Globals.write_acls and acls: rpout.write_acl(rpin.get_acl())
+	if rpin.isdir() and not rpout.isdir():
+		rpout.chmod(rpin.getperms() & 0777)
+	else: rpout.chmod(rpin.getperms())
+	if Globals.write_acls: rpout.write_acl(rpin.get_acl(), map_names = 0)
 	if not rpin.isdev(): rpout.setmtime(rpin.getmtime())
 
 def cmp_attribs(rp1, rp2):
@@ -222,7 +243,7 @@ def make_socket_local(rpath):
 
 	"""
 	assert rpath.conn is Globals.local_connection
-	s = socket.socket(socket.AF_UNIX)
+	s = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
 	try: s.bind(rpath.path)
 	except socket.error, exc:
 		raise SkipFileException("Socket error: " + str(exc))
@@ -265,9 +286,8 @@ class RORPath:
 		if self.index != other.index: return None
 
 		for key in self.data.keys(): # compare dicts key by key
-			if (key == 'uid' or key == 'gid') and self.issym():
+			if self.issym() and key in ('uid', 'gid', 'uname', 'gname'):
 				pass # Don't compare gid/uid for symlinks
-			elif key == 'perms' and not Globals.change_permissions: pass
 			elif key == 'atime' and not Globals.preserve_atime: pass
 			elif key == 'ctime': pass
 			elif key == 'devloc' or key == 'nlink': pass
@@ -281,7 +301,8 @@ class RORPath:
 				   not Globals.compare_inode or
 				   not Globals.preserve_hardlinks)): pass
 			elif (not other.data.has_key(key) or
-				  self.data[key] != other.data[key]): return None
+				  self.data[key] != other.data[key]):
+				return None
 		return 1
 
 	def equal_loose(self, other):
@@ -294,11 +315,7 @@ class RORPath:
 
 		"""
 		for key in self.data.keys(): # compare dicts key by key
-			if ((key == 'uid' or key == 'gid') and
-				(self.issym() or not Globals.change_ownership)):
-				# Don't compare gid/uid for symlinks, and only root
-				# can change ownership
-				pass
+			if key in ('uid', 'gid', 'uname', 'gname'): pass
 			elif (key == 'type' and self.isspecial() and
 				  other.isreg() and other.getsize() == 0):
 				pass # Special files may be replaced with empty regular files
@@ -306,7 +323,6 @@ class RORPath:
 			elif key == 'ctime': pass
 			elif key == 'devloc' or key == 'nlink': pass
 			elif key == 'size' and not self.isreg(): pass
-			elif key == 'perms' and not Globals.change_permissions: pass
 			elif key == 'inode': pass
 			elif key == 'ea' and not Globals.write_eas: pass
 			elif key == 'acl' and not Globals.write_acls: pass
@@ -314,6 +330,11 @@ class RORPath:
 				pass
 			elif (not other.data.has_key(key) or
 				  self.data[key] != other.data[key]): return 0
+
+		if self.lstat() and not self.issym() and Globals.change_ownership:
+			# Now compare ownership.  Symlinks don't have ownership
+			if user_group.map_rpath(self) != other.getuidgid(): return 0
+
 		return 1
 
 	def equal_verbose(self, other, check_index = 1,
@@ -325,11 +346,10 @@ class RORPath:
 			return None
 
 		for key in self.data.keys(): # compare dicts key by key
-			if ((key == 'uid' or key == 'gid') and
+			if (key in ('uid', 'gid', 'uname', 'gname') and
 				(self.issym() or not compare_ownership)):
 				# Don't compare gid/uid for symlinks, or if told not to
 				pass
-			elif key == 'perms' and not Globals.change_permissions: pass
 			elif key == 'atime' and not Globals.preserve_atime: pass
 			elif key == 'ctime': pass
 			elif key == 'devloc' or key == 'nlink': pass
@@ -428,6 +448,14 @@ class RORPath:
 		"""Return permission block of file"""
 		return self.data['perms']
 
+	def getuname(self):
+		"""Return username that owns the file"""
+		return self.data['uname']
+
+	def getgname(self):
+		"""Return groupname that owns the file"""
+		return self.data['gname']
+
 	def hassize(self):
 		"""True if rpath has a size parameter"""
 		return self.data.has_key('size')
@@ -623,8 +651,11 @@ class RPath(RORPath):
 	def setdata(self):
 		"""Set data dictionary using C extension"""
 		self.data = self.conn.C.make_file_dict(self.path)
-		if Globals.read_eas and self.lstat(): self.get_ea()
-		if Globals.read_acls and self.lstat(): self.get_acl()
+		if not self.lstat(): return
+		self.data['uname'] = self.conn.user_group.uid2uname(self.data['uid'])
+		self.data['gname'] = self.conn.user_group.gid2gname(self.data['gid'])
+		if Globals.read_eas: self.data['ea'] = self.conn.rpath.ea_get(self)
+		if Globals.read_acls: self.data['acl'] = self.conn.rpath.acl_get(self)
 		if Globals.read_resource_forks and self.isreg():
 			self.get_resource_fork()
 
@@ -885,6 +916,14 @@ class RPath(RORPath):
 			raise RPathException("Error closing file")
 		self.setdata()
 
+	def write_string(self, s, compress = None):
+		"""Write string s into rpath"""
+		assert not self.lstat(), "File %s already exists" % (self.path,)
+		outfp = self.open("wb", compress = compress)
+		outfp.write(s)
+		assert not outfp.close()
+		self.setdata()
+
 	def isincfile(self):
 		"""Return true if path looks like an increment file
 
@@ -993,28 +1032,22 @@ class RPath(RORPath):
 	def get_acl(self):
 		"""Return access control list object, setting if necessary"""
 		try: acl = self.data['acl']
-		except KeyError:
-			acl = eas_acls.AccessControlList(self.index)
-			if not self.issym(): acl.read_from_rp(self)
-			self.data['acl'] = acl
+		except KeyError: acl = self.data['acl'] = acl_get(self)
 		return acl
 
-	def write_acl(self, acl):
-		"""Change access control list of rp"""
-		acl.write_to_rp(self)
+	def write_acl(self, acl, map_names = 1):
+		"""Change access control list of rp
+
+		If map_names is true, map the ids in acl by user/group names.
+
+		"""
+		acl.write_to_rp(self, map_names)
 		self.data['acl'] = acl
 
 	def get_ea(self):
 		"""Return extended attributes object, setting if necessary"""
 		try: ea = self.data['ea']
-		except KeyError:
-			ea = eas_acls.ExtendedAttributes(self.index)
-			if not self.issym():
-				# Don't read from symlinks because they will be
-				# followed.  Update this when llistxattr,
-				# etc. available
-				ea.read_from_rp(self)
-			self.data['ea'] = ea
+		except KeyError: ea = self.data['ea'] = ea_get(self)
 		return ea
 
 	def write_ea(self, ea):
@@ -1060,4 +1093,10 @@ class RPathFileHook:
 		self.closing_thunk()
 		return result
 
-import eas_acls # Put at end to avoid regress
+
+# These two are overwritten by the eas_acls.py module.  We can't
+# import that module directory because of circular dependency
+# problems.
+def acl_get(rp): assert 0
+def ea_get(rp): assert 0
+
